@@ -4,7 +4,7 @@ Nick Mathewson
 """
 
 import unittest
-from test import support
+from test.support import os_helper
 
 import os
 import stat
@@ -145,34 +145,6 @@ class UUTest(unittest.TestCase):
         uu.encode(inp, out, filename)
         self.assertIn(safefilename, out.getvalue())
 
-    def test_no_directory_traversal(self):
-        relative_bad = b"""\
-begin 644 ../../../../../../../../tmp/test1
-$86)C"@``
-`
-end
-"""
-        with self.assertRaisesRegex(uu.Error, 'directory'):
-            uu.decode(io.BytesIO(relative_bad))
-        if os.altsep:
-            relative_bad_bs = relative_bad.replace(b'/', b'\\')
-            with self.assertRaisesRegex(uu.Error, 'directory'):
-                uu.decode(io.BytesIO(relative_bad_bs))
-
-        absolute_bad = b"""\
-begin 644 /tmp/test2
-$86)C"@``
-`
-end
-"""
-        with self.assertRaisesRegex(uu.Error, 'directory'):
-            uu.decode(io.BytesIO(absolute_bad))
-        if os.altsep:
-            absolute_bad_bs = absolute_bad.replace(b'/', b'\\')
-            with self.assertRaisesRegex(uu.Error, 'directory'):
-                uu.decode(io.BytesIO(absolute_bad_bs))
-
-
 class UUStdIOTest(unittest.TestCase):
 
     def setUp(self):
@@ -202,10 +174,11 @@ class UUStdIOTest(unittest.TestCase):
 class UUFileTest(unittest.TestCase):
 
     def setUp(self):
-        self.tmpin  = support.TESTFN_ASCII + "i"
-        self.tmpout = support.TESTFN_ASCII + "o"
-        self.addCleanup(support.unlink, self.tmpin)
-        self.addCleanup(support.unlink, self.tmpout)
+        # uu.encode() supports only ASCII file names
+        self.tmpin  = os_helper.TESTFN_ASCII + "i"
+        self.tmpout = os_helper.TESTFN_ASCII + "o"
+        self.addCleanup(os_helper.unlink, self.tmpin)
+        self.addCleanup(os_helper.unlink, self.tmpout)
 
     def test_encode(self):
         with open(self.tmpin, 'wb') as fin:
